
# q-expansion of newform 312.4.a.b, downloaded from the LMFDB on 15 January 2025.

# We generate the q-expansion using the Hecke eigenvalues a_p at the primes.
# Each a_p is given as a linear combination
# of the following basis for the coefficient ring.

def make_data():

    from sage.all import prod, floor, prime_powers, gcd, QQ, primes_first_n, next_prime, RR

    def discrete_log(elts, gens, mod):
        # algorithm 2.2, page 16 of https://arxiv.org/abs/0903.2785
        def table_gens(gens, mod):
            T = [1]
            n = len(gens)
            r = [None]*n
            s = [None]*n
            for i in range(n):
                beta = gens[i]
                r[i] = 1
                N = len(T)
                while beta not in T:
                    for Tj in T[:N]:
                        T.append((beta*Tj) % mod)
                    beta = (beta*gens[i]) % mod
                    r[i] += 1
                s[i] = T.index(beta)
            return T, r, s
        T, r, s = table_gens(gens, mod)
        n = len(gens)
        N = [ prod(r[:j]) for j in range(n) ]
        Z = lambda s: [ (floor(s/N[j]) % r[j]) for j in range(n)]
        return [Z(T.index(elt % mod)) for elt in elts]
    def extend_multiplicatively(an):
        for pp in prime_powers(len(an)-1):
            for k in range(1, (len(an) - 1)//pp + 1):
                if gcd(k, pp) == 1:
                    an[pp*k] = an[pp]*an[k]
    from sage.all import PolynomialRing, NumberField, ZZ
    R = PolynomialRing(QQ, "x")
    f = R(poly_data)
    K = NumberField(f, "a")
    betas = [K([c/ZZ(den) for c in num]) for num, den in basis_data]
    convert_elt_to_field = lambda elt: sum(c*beta for c, beta in zip(elt, betas))
    # convert aps to K elements
    primes = primes_first_n(len(aps_data))
    good_primes = [p for p in primes if not p.divides(level)]
    aps = map(convert_elt_to_field, aps_data)
    if not hecke_ring_character_values:
        # trivial character
        char_values = dict(zip(good_primes, [1]*len(good_primes)))
    else:
        gens = [elt[0] for elt in hecke_ring_character_values]
        gens_values = [convert_elt_to_field(elt[1]) for elt in hecke_ring_character_values]
        char_values = dict([(
            p,prod(g**k for g, k in zip(gens_values, elt)))
            for p, elt in zip(good_primes, discrete_log(good_primes, gens, level))
            ])
    an_list_bound = next_prime(primes[-1])
    an = [0]*an_list_bound
    an[1] = 1
    
    from sage.all import PowerSeriesRing
    PS = PowerSeriesRing(K, "q")
    for p, ap in zip(primes, aps):
        if p.divides(level):
            euler_factor = [1, -ap]
        else:
            euler_factor = [1, -ap, p**(weight - 1) * char_values[p]]
        k = RR(an_list_bound).log(p).floor() + 1
        foo = (1/PS(euler_factor)).padded_list(k)
        for i in range(1, k):
            an[p**i] = foo[i]
    extend_multiplicatively(an)
    return PS(an)
level = 312
weight = 4
poly_data = [-55, 0, 1]

# The entries in the following list give a basis for the
# coefficient ring in terms of a root of the defining polynomial above.
# Each line consists of the coefficients of the numerator, and a denominator.
basis_data  = [[[1, 0], 1], [[0, 2], 1]]

hecke_ring_character_values = None
aps_data = [[0, 0], [-3, 0], [-2, 1], [10, 1], [-10, 2], [-13, 0], [-34, -2], [14, -1], [60, 4], [-6, 0], [230, -3], [114, 2], [46, -11], [216, -10], [358, 8], [-178, 16], [78, 0], [-102, 48], [102, -51], [-302, -6], [242, -22], [880, 20], [-134, -20], [38, -93], [2, 54], [-1066, 38], [772, 78], [-1048, -48], [362, -28], [-42, 20], [-292, 74], [-168, 36], [-1682, -43], [1876, -4], [-250, 135], [18, -41], [-618, -116], [354, -23], [-626, -176], [1302, 162], [-3140, 44], [-1218, -212], [-384, -164], [4194, 4], [-1154, -203], [3884, -110], [-236, -288], [-2658, 121], [410, -342], [-2534, -76], [3602, -190], [-1310, 242], [2306, 138], [2368, 100], [3002, 6], [-68, 4], [346, -528], [1122, 353], [3910, 224], [3406, -35], [24, -434], [4190, -365], [730, -309], [-516, -484], [2150, 40], [2022, 181], [-3946, -359], [646, -508], [-76, -188], [538, 494], [3150, 197], [2222, 184], [-3576, 408], [-7342, 0], [-8962, 187], [1578, -348], [-7806, 366], [-11534, 186], [11646, -9], [-6110, -482], [1992, -276], [-3734, -812], [-9846, 388], [3734, -280], [-6468, -578], [-6608, -352], [1454, -749], [-7318, 338], [-7178, -607], [10738, 173], [8120, -124], [-7866, -342], [6646, 709], [-1592, 20], [-7914, 537], [17644, 88], [-15658, -255], [3726, -84], [7836, 452], [1826, 832], [-12180, 612], [4590, 151], [17652, -144], [-15270, 358], [4408, 1194], [874, 492], [6082, -844], [14182, 151], [-84, -56], [-8410, -1328], [8336, 84], [10362, -774], [8038, 13], [17490, -821], [8762, -1049], [12094, -314], [-11642, 1053], [10088, 1324], [-9934, -596], [-764, 564], [12714, 134], [-2110, 316], [-5970, 1092], [13998, -186], [-3098, -87], [-5726, 1414], [20650, 96], [3308, 1192], [19020, -494], [24394, -216], [8226, -89], [-17758, 1026], [-2988, 458], [-7926, -832], [-2810, 2081], [-9046, 1736], [26654, -101], [-2814, -369], [30850, -158], [-10914, -1356], [6870, 299], [-7522, -1705], [11972, 1826], [20414, -822], [-4750, 1480], [-29202, -570], [11962, 534], [-38742, -450], [-11156, -500], [-2798, -2768], [-6726, 998], [-1326, -1842], [17744, 550], [5928, -1008], [-4216, -358], [-19256, 712], [-22056, 1260], [-32674, 509], [438, 64], [12638, -813], [-17170, -404], [18394, -910], [1110, 3209], [50884, 332], [7998, 279], [15510, 1874], [-15384, -1412], [-10786, -3120], [-22870, -450], [1502, 358], [24346, -670], [59202, -150], [-4652, -1416], [8866, -3418], [3868, 1378], [-14686, 1408], [27698, 2235], [-14994, -117], [12148, -234], [-6390, 2680], [23434, 2037], [51916, -428], [22058, -1896], [17902, 2139], [-7842, 1152], [13842, -1578], [36374, -1596], [-10738, 1179], [8122, -3322], [-37026, -432], [-20022, -196], [9482, -712], [13348, 1216], [-23530, 1751], [36692, -1644], [27466, -1030], [-24262, -444], [-8998, 1080], [-24826, 2707], [10328, 500], [11990, -1735], [24280, 2316], [-40230, -2600], [-11846, 3076], [646, 2476], [30798, -492], [38118, -623], [5776, -1004], [-16370, 3587], [-11632, -2754], [-62250, 2072], [77702, -636], [-6216, 1440], [-54610, -514], [8410, -1168], [-46238, -1464], [23688, -2080], [10914, 3252], [32394, -3054], [-24242, -3037], [-52546, 456], [9586, -4935], [-15658, -3481], [-89178, -395], [11424, 5400], [3974, -4460], [-5646, -2920], [-18780, -3596], [-71308, 158], [29774, -520], [54998, -1244], [-25268, -2648], [17222, -917], [-34938, -4338], [-25844, 4456], [-39822, 3338], [-10038, -2618], [16262, -1265], [-25796, 2756], [53664, -408], [-62926, -2444], [24912, 890], [56712, 548], [32170, -2838], [-11954, 6023], [-31908, 5340], [8330, 5493], [-58682, 2424], [-12810, 4343], [-7956, 5096], [-45630, -1264], [34542, 1461], [-1594, -5422], [61294, -1124], [51930, 156], [-59634, -1930], [-85246, 1800], [15334, 3407], [52482, 94], [-56302, 976], [2052, -4402], [50100, -2964], [-13422, -2604], [-12874, -6504], [82334, -3069], [-16404, -4352], [-23514, 2733], [-56018, 797], [31322, -2465], [62582, 5470], [36250, 4564], [-14498, -3423], [52394, 2386], [-123236, -1290], [-79970, -456], [42814, -4065], [-1502, 1982], [38410, -1222], [117418, 220], [-85944, 2052], [-42784, -1436], [135410, 717], [64968, 3708], [73410, 22], [14202, 5219], [-21480, 1644], [74918, 48], [30630, 5575], [103770, 855], [47986, 1394], [-9130, -2084], [-7476, -12], [140414, 639], [31670, 920], [63654, 1708], [-122230, -422], [61280, -7360], [129570, 1998], [65820, 3404], [-64470, 2243], [-30694, -2136], [-79170, -1839], [48020, 6754], [-82460, -2656], [41036, 1976], [-24214, -4662], [-852, -5968], [-778, -1200], [-102586, 442], [97298, 5420], [-48972, 3588], [-52978, 2405], [-52418, 500], [57548, 1444], [38222, -6484], [33062, 4640], [68226, -5358], [19698, 1884], [97810, -3494], [11062, 10670], [25080, -11210], [48082, 6780], [96382, 3728], [-11206, -897], [68350, 5435], [19734, 3640], [-2014, 9561], [-85426, -4835], [28168, 7148], [-25934, 436], [-104486, 5280], [-24822, 8340], [39392, 2600], [-58562, -6682], [90990, 1987], [-41446, 4928], [-8614, 130], [82646, 8701], [-108054, 3770], [11932, -6474], [96490, 6406], [-59082, -4464], [-120770, -5381], [-45260, -2326], [-79378, -6967], [72212, -2536], [-116958, -5600], [-45358, -8747], [49446, -2664], [-98294, -2718], [59846, -4035], [-19246, 5882], [-129786, 4283], [-66860, -5538], [8274, 2660], [67622, -1212], [61790, 3632], [151442, -5482], [-107046, 82], [52626, 2092], [-38606, -9002], [-113642, -5806], [10796, 2104], [43234, -9418], [-17248, 6718], [794, 2228], [-138666, -1339], [90362, 5107], [-90358, 5348], [83712, 5696], [144664, 4518], [-22122, -748], [119270, -3816], [-127864, 6428], [7718, 5384], [63978, 4488], [74544, -2900], [-58774, 9172], [-74318, -9436], [-590, 5472], [-81762, 7479], [1022, 3489], [-117790, -3915], [87246, -2205], [218578, -1359], [-190258, 1226], [83678, -6461], [49018, 7760], [-37978, 603], [17596, -14712], [160202, 5158], [-163722, 1167], [-48146, -8966], [-68172, 13468], [118550, -7608], [59230, -5008], [-15522, -11533], [-78762, 7722], [96636, -3244], [98262, 6429], [123250, -1874], [82422, 8678], [140330, -4519], [-160754, -5281], [30822, -1179], [-4928, 11788], [-742, 18192], [-79762, 6735], [-17422, -765], [135318, -1730], [113602, -4472], [150158, -7900], [-226716, 2172], [-31368, -7934], [-258182, -964], [3350, -3628], [31338, 13850], [-30864, -15460], [245174, -689], [196060, -5920], [-39178, 8286], [7986, 10766], [111186, 2094], [-141288, -588], [60186, -14740], [-58346, 12875], [-202480, -1636], [-76914, -5153], [-18350, -9963], [-16936, 9256]]
